"""yiban"""
from .Apis import *

__title__ = 'yiban'
__description__ = 'Apis lib for yiban.'
__url__ = 'https://github.com/sricor/yiban'
__version__ = '1.0.3'
__author__ = 'joSricor'
__author_email__ = 'josricor@outlook.com'
__ide__ = 'PyCharm - https://www.jetbrains.com/pycharm/'
